import numpy as np
import pytest
from surropt.core.utils import is_row_member

_b = np.array([[1, 4.], [3, 6], [5, 9], [2, 8]])
_a1 = np.array([1., 3])
_a2 = np.array([5., 9])
_a3 = np.array([8., 2])
_a4 = np.array([3., 6])
_a5 = np.vstack((_a1, _a4))
_a6 = np.vstack((_a3, _a1))
_a7 = np.vstack((_a2, _a3))
_a8 = np.vstack((_a2, _a2))
_a9 = np.vstack((_a4, _a2))
_is_row_member_array_list = [(_a1, _b, False),
                             (_a2, _b, True),
                             (_a3, _b, False),
                             (_a4, _b, True),
                             (_a5, _b, np.array([False, True])),
                             (_a6, _b, np.array([False, False])),
                             (_a7, _b, np.array([True, False])),
                             (_a8, _b, np.array([True, True])),
                             (_a9, _b, np.array([True, True]))]


@pytest.mark.parametrize("a,b,correct_result", _is_row_member_array_list)
def test_is_row_member(a, b, correct_result):
    if a.ndim == 1:
        assert is_row_member(a, b) == correct_result
    else:
        assert np.array_equal(is_row_member(a, b), correct_result)



import numpy as np
import pytest
from scipy.io import loadmat

from src.surropt.utils.models import evaporator
from tests_ import RESOURCES_PATH


# fixture
@pytest.fixture
def load_mat_file():
    mat_contents = loadmat(RESOURCES_PATH / "evap53pts.mat")
    return mat_contents['doeBuild']


def test_evaporator_model(load_mat_file):
    check = []

    n_samples = load_mat_file.shape[0]

    for i in range(n_samples):
        # inputs
        f1_lhs, f3_lhs, p100_lhs, f200_lhs = load_mat_file[i][3:7].tolist()

        # results from matlab
        res = dict(zip(["f2", "f4", "f5", "x2", "t2", "t3", "p2", "f100",
                        "t100", "q100", "t201", "q200"],
                       load_mat_file[i][7:19].tolist()))

        # python model call
        sol = evaporator(f1=f1_lhs, f3=f3_lhs, p100=p100_lhs, f200=f200_lhs)

        check.append(all(np.allclose(res[k], sol['extras'][k])
                         for k in res.keys()))

    assert all(check), "Evaporator model failed in some cases."



import numpy as np
import pytest
from scipy.io import loadmat

from surropt.utils.models import evaporator
from surropt.caballero import Caballero
from tests_ import RESOURCES_PATH


# fixture
@pytest.fixture
def load_mat_file():
    mat_contents = loadmat(RESOURCES_PATH / "evap53pts.mat")
    return mat_contents['doeBuild']


def test_caballero(load_mat_file):
    # load input design
    x = load_mat_file[:, 3:7]

    # constraints
    x2_raw = load_mat_file[:, 10]
    p2_raw = load_mat_file[:, 13]

    g1 = 35.5 - x2_raw
    g2 = p2_raw - 80.0
    g3 = 40.0 - p2_raw

    g = np.c_[g1, g2, g3]

    # objective function
    f = load_mat_file[:, 19]

    # sampling function
    model_function = evaporator

    # bounds
    lb = [8.5, 0, 102, 0]
    ub = [20., 100., 400., 400.]

    caballero_obj = Caballero(x=x, g=g, f=f, model_function=model_function,
                              lb=lb, ub=ub, regression='poly1')

    caballero_obj.optimize()
    pass



import numpy as np
from scipy.io import loadmat

from surropt.utils.models import evaporator
from surropt.caballero import Caballero
from tests_ import RESOURCES_PATH
from surropt.core.options.nlp import DockerNLPOptions
from surropt.core.procedures.output import Report


def main():
    mat_contents = loadmat(RESOURCES_PATH / "evap53pts.mat")
    mat_contents = mat_contents['doeBuild']

    # load input design
    x = mat_contents[:, 3:7]

    # constraints
    x2_raw = mat_contents[:, 10]
    p2_raw = mat_contents[:, 13]

    g1 = 35.5 - x2_raw
    g2 = p2_raw - 80.0
    g3 = 40.0 - p2_raw

    g = np.c_[g1, g2, g3]

    # objective function
    f = mat_contents[:, 19]

    # sampling function
    def model_function(x):
        return evaporator(f1=x[0], f3=x[1], p100=x[2], f200=x[3])

    # bounds
    lb = [8.5, 0, 102, 0]
    ub = [20., 100., 400., 400.]

    # nlp server options
    nlp_opts = DockerNLPOptions(name='wsl-server',
                                server_url='http://localhost:5000')

    # report options object
    # this is the object where output generated by the caballero procedure is
    # reported to the user via terminal or plot window
    report_opts = Report(terminal=True, plot=False)

    caballero_obj = Caballero(x=x, g=g, f=f, model_function=model_function,
                              lb=lb, ub=ub, regression='poly1',
                              nlp_options=nlp_opts)

    caballero_obj.optimize()


if __name__ == "__main__":
    main()



import numpy as np
import pytest

from surropt.caballero.problem import is_inside_hypercube

# tuple follows: (point, lower bound, upper bound, correct result)
_array_list = [([9.429, 24.721, 400, 217.738],
                [8.5, 0.0, 102.0, 0.0],
                [20.0, 100.0, 400.0, 400.0], False),
               ([-3, -3], [-5, -4], [-2, -1], True),
               ([-5.1, -3], [-5, -4], [-2, -1], 'Error'),
               ([5e-7, 1.1e-6], [0, 0], [1e-6, 1e-6], 'Error'),
               ([5e-7, 1e-6], [0, 0], [1e-6, 1e-6], False),
               ([5e-7, 5e-7], [0, 0], [1e-6, 1e-6], True)]


@pytest.mark.parametrize("p,lb,ub, correct_result", _array_list)
def test_is_inside(p, lb, ub, correct_result):
    p = np.array(p)
    lb = np.array(lb)
    ub = np.array(ub)

    if type(correct_result) is bool:
        assert is_inside_hypercube(p, lb, ub) == correct_result
    else:
        with pytest.raises(ValueError):
            is_inside_hypercube(p, lb, ub)



import numpy as np
from scipy.io import loadmat

from surropt.caballero import Caballero
from surropt.core.options.nlp import DockerNLPOptions
from surropt.core.procedures.output import Report
from tests_ import RESOURCES_PATH

from win32com.client import Dispatch


def open_connection():
    aspen = Dispatch('Apwn.Document')
    aspen.InitFromArchive2(str(RESOURCES_PATH / 'infill.bkp'))
    return aspen


def aspen_auto(x: np.ndarray, asp_srv):
    # set input values
    asp_srv.Tree.FindNode(r"\Data\Blocks\TOWER\Input\BASIS_RR").Value = x[0]
    asp_srv.Tree.FindNode(r"\Data\Blocks\TOWER\Input\D:F").Value = x[1]

    # run the results
    asp_srv.Engine.Run2()

    # convergence flag
    conv = asp_srv.Tree.FindNode(
        r"\Data\Results Summary\Run-Status\Output\UOSSTAT2").Value

    if conv == 8:
        # run converged, sample results
        status = True

        RR = asp_srv.Tree.FindNode("\Data\Blocks\TOWER\Output\MOLE_RR").Value
        DF = asp_srv.Tree.FindNode("\Data\Blocks\TOWER\Output\MOLE_DFR").Value
        D = asp_srv.Tree.FindNode("\Data\Streams\D\Output\TOT_FLOW").Value
        XB = asp_srv.Tree.FindNode(
            "\Data\Streams\B\Output\MOLEFRAC\MIXED\PROPENE").Value
        B = asp_srv.Tree.FindNode("\Data\Streams\B\Output\TOT_FLOW").Value
        QR = asp_srv.Tree.FindNode(
            "\Data\Blocks\TOWER\Output\REB_DUTY").Value * 4.1868
        L = asp_srv.Tree.FindNode("\Data\Blocks\TOWER\Output\MOLE_L1").Value
        V = asp_srv.Tree.FindNode("\Data\Blocks\TOWER\Output\MOLE_VN").Value
        F = asp_srv.Tree.FindNode("\Data\Streams\FEED\Output\TOT_FLOW").Value
        XD = asp_srv.Tree.FindNode(
            "\Data\Streams\D\Output\MOLEFRAC\MIXED\PROPENE").Value

        J = -(20 * D + (10 - 20 * XB) * B - 70 * QR)

        g = [0.995 - XD, QR - 80.0]

    else:
        status = False
        J = 0
        g = [1.0, 1.0]  # value above con_tol to ensure infeasibility

    results = {
        'status': status,
        'f': J,
        'g': g,
        'extras': []
    }
    return results


def main():
    # load initial doe
    m_c = loadmat(RESOURCES_PATH / 'column33pts.mat')

    # MVs
    x = m_c['doeBuild'][:, :2]

    # constraints
    xd_raw = m_c['doeBuild'][:, 4]
    qr_raw = m_c['doeBuild'][:, 7]

    g1 = 0.995 - xd_raw
    g2 = qr_raw - 80.0

    g = np.c_[g1, g2]

    # objective function
    f = m_c['doeBuild'][:, 6]

    # open the connection
    aspen = open_connection()

    # sampling function
    def model_function(x): return aspen_auto(x, asp_srv=aspen)

    # bounds
    lb = [7.0, 0.1]
    ub = [25.0, 0.9]

    # nlp server options
    nlp_opts = DockerNLPOptions(name='wsl-server',
                                server_url='http://localhost:5000')

    # report options object
    # this is the object where output generated by the caballero procedure is
    # reported to the user via terminal or plot window
    report_opts = Report(terminal=True, plot=False)

    caballero_obj = Caballero(x=x, g=g, f=f, model_function=model_function,
                              lb=lb, ub=ub, regression='poly1',
                              nlp_options=nlp_opts,
                              report_options=report_opts)

    caballero_obj.optimize()


if __name__ == "__main__":
    main()
